
package com.ibm.portal.portlet.service.contentaccess;

import com.ibm.portal.portlet.service.PortletService;

import java.net.*;
import java.io.*;

import javax.portlet.*;

/**
 * Service to access external and internal resources.
 *
 * This on-demand proxy allows the creation of a URL or Stream 
 * which can then be passed to other classes without knowing 
 * about the proxy support to enable seemless adaption to 
 * proxied environments.  
 * 
 * The proxying will be provided to the URL object by means of a StreamHandler.
 * 
 * @ibm-api
 * @since 5.1
 * @ibm-java2security
 */

public interface ContentAccessService extends PortletService
{
    
    /**
     * Returns an URL object after following redirects
     * using a proxy if necessary
     * 
     * @param urlString a string representing the start url
     * 
     * @param request   the portlet request
     * 
     * @param response  the portlet response
     * 
     * @return URL object based on the given url string and the proxy configuration
     * 
     * @exception MalformedURLException
     *                   if the given urlString is a malformed URL.
     * @exception IOException
     *                   if a required credential could not be retrieved
     *                   or the expected URL could not been created.
     */
    public URL getURL(String urlString, 
                      PortletRequest request,
                      PortletResponse response) throws IOException;

    /**
     * Returns an input stream that reads from the passed url.
     * 
     * @param urlString a string representing the start url
     * 
     * @param req
     * 
     * @param resp
     * 
     * @return InputStream from the proxied connection
     * 
     * @exception MalformedURLException
     *                   thrown if the URL is invalid
     * @exception IOException
     *                   thrown if the portlet service fails
     */
    public InputStream getInputStream(String urlString,
                        PortletRequest req,
                        PortletResponse resp) throws IOException;

    /**
     * Returns the markup of the input stream
     * read from the passed url.
     * 
     * @param urlString a string representing the start url
     * 
     * @param req
     * 
     * @param resp
     * 
     * @return String representing the markup of the proxied connection
     * 
     * @exception MalformedURLException
     *                   thrown if the URL is invalid.
     * @exception IOException
     *                   thrown if the portlet service fails.
     */
    public String getMarkup(String urlString,
                        PortletRequest req,
                        RenderResponse resp) throws IOException;

    /**
     * Writes the content of the given url to the output stream of the 
     * portlet following redirects and using a proxy if necessary.
     * If the URL is relative and a request dispatcher is given, the service
     * first attempts to include it using the request dispatcher, to handle
     * local servlets or JSPs
     * 
     * @param urlString a string representing the url
     * 
     * @param request   the portlet request
     * 
     * @param response  the portlet response
     * 
     * @param context   the context for including relative URLs, may be null
     * 
     * @exception MalformedURLException
     *                   thrown if the URL is invalid.
     * @exception IOException
     *                   thrown if the portlet service fails.
     */
    public void include(String urlString,
                        RenderRequest request,
                        RenderResponse response,
                        PortletContext context) throws IOException;
  
}

