/*
********************************************************************
* IBM Confidential                                                 *
*                                                                  *
* OCO Source Materials                                             *
*                                                                  *
* 5724-E76 and 5724-E77                                            *
*                                                                  *
* (C) Copyright IBM Corp. 2001, 2003                               *
*                                                                  *
* The source code for this program is not published or otherwise   *
* divested of its trade secrets, irrespective of what has been     *
* deposited with the U.S. Copyright Office.                        *
********************************************************************
*/

package com.ibm.portal.portlet.service.credentialvault;

import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.portlet.PortletRequest;
import javax.security.auth.Subject;

import com.ibm.portal.ObjectID;
import com.ibm.portal.portlet.service.PortletService;
import com.ibm.portal.portlet.service.credentialvault.credentials.Credential;

/**
 * Service that allows to retrieve credentials from the credential vault and to manage vault segments and slots.
 * Service interface for JSR-compliant portlets.
 * 
 * @ibm-api
 * @since 5.1
 * @ibm-java2security 
 **/
public interface CredentialVaultService extends PortletService
{
    // Used for the super class only, do not use this for creating slots
    int SECRET_TYPE_UNDEFINED                     = com.ibm.wps.portletservice.credentialvault.CredentialVaultService.SECRET_TYPE_UNDEFINED;
    // The secret type is userid / password
    int SECRET_TYPE_USERID_STRING_PASSWORD_STRING = com.ibm.wps.portletservice.credentialvault.CredentialVaultService.SECRET_TYPE_USERID_STRING_PASSWORD_STRING;
    // The secret is given in the form of a byte array
    int SECRET_TYPE_BYTEARRAY                     = com.ibm.wps.portletservice.credentialvault.CredentialVaultService.SECRET_TYPE_BYTEARRAY;
    // The secret is a Java Object
    int SECRET_TYPE_JAVA_OBJECT                   = com.ibm.wps.portletservice.credentialvault.CredentialVaultService.SECRET_TYPE_JAVA_OBJECT;
    // There is no secret data provided
    int SECRET_TYPE_NO_SECRET_DATA                = com.ibm.wps.portletservice.credentialvault.CredentialVaultService.SECRET_TYPE_NO_SECRET_DATA;
    // The secret is given in the form of a Jaas Subject
    int SECRET_TYPE_JAAS_SUBJECT                  = com.ibm.wps.portletservice.credentialvault.CredentialVaultService.SECRET_TYPE_JAAS_SUBJECT;
    
    static String PREDEFINED_SLOT_USER_JAAS_SUBJECT = CredentialVaultService.PREDEFINED_SLOT_USER_JAAS_SUBJECT;

    /**
     *  Returns an Iterator of all Credential Types that are registered in the Credential Type Registry
     *
     *  @return   Iterator of String objects
     *            All Credential Types that are registered in the Credential Type Registry
     **/
    Iterator getCredentialTypes();
    

    /**
     *  Returns a Credential Slot's description for the specified locale.
     *
     *  @param   slotId
     *           The credential (slot) id.
     *
     *  @param   locale
     *           The description locale. If set to <code>null</code>, the default locale will be used.
     *
     *  @return   String
     *            The credential slot description for the specified locale.
     *  @exception   CredentialVaultException
     *               if the description could not been retrieved.
     **/
    String getCredentialSlotDescription(String slotId, Locale locale) throws CredentialVaultException;
    
    
    /**
     *  Returns a Iterator of all credential slots that a portlet is authorized to use
     *
     *  @param   request
     *           The portlet request is needed by the CredentialVault service in order to determine 
     *           information about the portlet.
     *
     *  @return   Iterator of CredentialSlotConfig
     *            All Credential Slots that a portlet is authorized to use
     *  @exception   CredentialVaultException      
     *               if the list of slots could not been retrieved.
     **/
    Iterator getAccessibleSlots(PortletRequest request) throws CredentialVaultException;
    

    /**
     *  Sets a credential's binary secret.
     *
     *  @param   slotId
     *           The credential (slot) id.
     *
     *  @param   secret
     *           The credential secret data in binary form.
     *
     *  @param   portletRequest
     *           The portlet request is used to determine parameters like the portlet id and user id.
     *
     *  @exception   CredentialVaultException
     *               if the credential secret is not of the type binary or if the secret could not be set.
     **/
    void setCredentialSecretBinary(String slotId, byte[] secret, PortletRequest portletRequest) 
    throws CredentialVaultException;
    

    /**
     *  Sets a credential's user/password secret.
     *
     *  @param   slotId
     *           The credential (slot) id.
     *
     *  @param   userId
     *           The credential's userId.
     *
     *  @param   password
     *           The credential's password.
     *
     *  @param   portletRequest
     *           The portlet request is used to determine parameters like the portlet id and user id.
     *
     *  @exception   CredentialVaultException
     *               if the credential secret is not of the type user/password or if the secret could not be set.
     **/
    void setCredentialSecretUserPassword(String slotId, String userId, char[] password, PortletRequest portletRequest) 
    throws CredentialVaultException;
    

    /**
     * Creates a new Credential Slot.
     * This method should be used by "ordinary" portlets, but not by "admin portlets".
     *
     * @param resourceName
     *        Name of the resource
     * @param segmentId
     *        Object id of the segment that this slot is created in
     * @param descriptions
     *        The slot descriptions as a <code>Map</code>, keyed by their corresponding <code>Locale</code> objects
     *        Key:   <code>Locale</code> (max length 64 characters)
     *        Value: <code>String</code> (max length 255 characters)
     * @param keywords
     * 	      Sets the keywords that can be used to search for this slot (max length 1024 characters)
     * @param secretType
     *        The secrtet type identifier
     * @param active
     *        Flag whether this credential may only be returned in form of an active credential object (<code>true</code>)
     *        or both as active and passive credential objects (<code>false</code>)
     * @param portletPrivate
     *        Flag whether the credential secret is portlet secific (<code>true</code>) 
     *        or shared betwen all of a user's portlets (<code>false</code>).
     * @param portletRequest
     *        The portlet request.
     * @return CredentialSlot
     *         The Credential Slot configuration
     **/
    public CredentialSlotConfig createCredentialSlot
    ( String resourceName, ObjectID segmentId, Map descriptions, Map keywords, 
      int secretType, boolean active, boolean portletPrivate, PortletRequest portletRequest)
    throws CredentialVaultException;

   /**
    * Deletes an existing Credential Slot.
    *
    *  @param   slotName
    *           The credential (slot) id.
    *  @exception   CredentialVaultException
    *               if the credential slot could not be deleted.
    **/
    public void deleteCredentialSlot( String slotName ) throws CredentialVaultException;

    /**
     * Returns the ObjectID of the default user managed vault segment.
     * Currently there is only one user-managed segment, so this returns the ID of <i>the</i> user managed segment.
     *
     * @return ObjectID
     *         The ObjectID of the default user managed vault segment.
     **/
    ObjectID getDefaultUserCredentialSegmentId() throws CredentialVaultException;

    /**
     * Returns a List of all Vault Segments.
     *
     * @return List of VaultSegmentConfig 
     *         All vault segments.
     **/
    List getAllCredentialSegments() throws CredentialVaultException;

    /**
     *  Returns a specific credential.
     * 
     *  @param   slotId
     *           The credential (slot) id.
     *
     *  @param   type
     *           The credential type as specified in the credential registry.
     *
     *  @param   config
     *           The backend application specific configuration that is needed to initialize this credential. 
     *           This is one part of the credential configuration. The credential vault service will usually 
     *           add further information to this config from other sources: the user's secret from the actual credential store
     *           and the credential instace specific parameters from the portal's credential configuration.
     *
     *  @param   request
     *           The portlet request is used to determine information about the portlet.
     *
     *  @return   Credential
     *            The credential.
     *
     *  @exception   CredentialVaultException
     *               if the credential could not been retrieved - either for technical or secuity reasons.
     *
     *  @exception   CredentialSecretNotSetException
     *               if the requested credential secret is not set (by the user or admin).
     **/
    Credential getCredential(String slotId, String type, Map config, PortletRequest request) 
    throws CredentialVaultException;

    /**
     *  Returns the user's JAAS subject. This is a special case of the getCredential call.
     * 
     *  @param   request
     *           The portlet request is used to determine the user.
     *
     *  @return   Subject
     *            The user's JAAS subject.
     *
     *  @exception   CredentialVaultException
     *               if the subject could not been retrieved.
     **/
    Subject getUserSubject(PortletRequest request) throws CredentialVaultException;

}
