/*
********************************************************************
* IBM Confidential                                                 *
*                                                                  *
* OCO Source Materials                                             *
*                                                                  *
* 5724-E76 and 5724-E77                                            *
*                                                                  *
* (C) Copyright IBM Corp. 2001, 2003                               *
*                                                                  *
* The source code for this program is not published or otherwise   *
* divested of its trade secrets, irrespective of what has been     *
* deposited with the U.S. Copyright Office.                        *
********************************************************************
*/
package com.ibm.wps.portlets.menu;

import java.io.InputStream;
import java.io.InputStreamReader;

import org.apache.jetspeed.portlet.PortletAdapter;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletSettings;

import com.ibm.wps.portlet.menu.MenuContext;
import com.ibm.wps.portlet.menu.MenuProvider;
import com.ibm.wps.portlet.menu.MenuTree;
import com.ibm.wps.portletservice.portletmenu.LogMenuService;
import com.ibm.wps.portletservice.portletmenu.XMIMenuService;

/**
 * {@link XMIMenuTreePortlet} is a convenience implementation of a portlet that exposes a
 * static portlet menu based on the content of an XMI file. 
 * 
 * <P>The {@link XMIMenuTreePortlet} is a default implementation for the
 * {@link org.apache.jetspeed.portlet.Portlet} interface. If a portlet uses a static XMI tree,
 * it is recommended that it derives from {@link XMIMenuTreePortlet}.
 * The tree structure is completely defined in an XML file, e.g.,
 * <CODE>menu-tree.xml</CODE>. In the deployment descriptor <CODE>portlet.xml</CODE>,
 * the configuration parameter <CODE>XMIFilePath</CODE> for the concrete portlet
 * specifies the full path of this XML file relative to the directory
 * <CODE>&lt;Portlet_HOME&gt;/war</CODE>. The method {@link XMIMenuTreePortlet#getMenu}
 * does not have to be implemented any more.</P>
 * 
 * <P>The following example shows a section of <CODE>portlet.xml</CODE> including the
 * configuration parameter for <CODE>XMIFilePath</CODE>.</P>
 * 
 * <PRE>
 * &lt;concrete-portlet href="#Portlet_1"&gt;
 * &lt;portlet-name&gt; ... &lt;/portlet-name&gt;
 * 
 *     &lt;config-param&gt;
 *         &lt;param-name&gt;XMIFilePath&lt;/param-name&gt;
 *         &lt;param-value&gt;menu-tree.xml&lt;/param-value&gt;
 *     &lt;/config-param&gt;
 * 
 * &lt;/concrete-portlet&gt;
 * </PRE>
 * 
 * <P>The next example shows how <CODE>menu-tree.xml</CODE> can be implemented
 * to create a menu tree with two nodes and two subnodes of the first node.
 * Title and description are set in two different locales.</P>
 * 
 * <P>Code of <CODE>menu-tree.xml</CODE>:</P>
 * 
 * <PRE>
 * &lt;!DOCTYPE menu-tree PUBLIC "-//IBM//DTD Portlet Menu 1.0//EN" "menu-tree.dtd"&gt;
 * &lt;!-- root node --&gt;
 * &lt;menu-tree id="id0"&gt;
 *     &lt;!-- root node --&gt;
 *     &lt;menu-node id="rootId"&gt;
 *         &lt;display-info&gt;
 *             &lt;locale&gt;en&lt;/locale&gt;
 *             &lt;title&gt;XMI Test Portlet&lt;/title&gt;
 *             &lt;description&gt;XMI Tree Description&lt;/description&gt;
 *         &lt;/display-info&gt;
 *         &lt;display-info&gt;
 *             &lt;locale&gt;fr&lt;/locale&gt;
 *             &lt;title&gt;XMI Test Portlet&lt;/title&gt;
 *             &lt;description&gt;XMI Tree Description&lt;/description&gt;
 *         &lt;/display-info&gt;
 *         &lt;!-- node 1 --&gt;
 *         &lt;menu-node id="id1"&gt;
 *             &lt;display-info&gt;
 *                 &lt;locale&gt;en&lt;/locale&gt;
 *                 &lt;title&gt;Node 1&lt;/title&gt;
 *                 &lt;description&gt;Node 1 Description&lt;/description&gt;
 *             &lt;/display-info&gt;
 *             &lt;display-info&gt;
 *                 &lt;locale&gt;fr&lt;/locale&gt;
 *                 &lt;title&gt;Noeud 1&lt;/title&gt;
 *                 &lt;description&gt;Noeud 1 Description&lt;/description&gt;
 *             &lt;/display-info&gt;
 *             &lt;action-name&gt;Node1Action&lt;/action-name&gt;
 *             &lt;param id="n1-pid1"&gt;
 *                 &lt;param-name&gt;Param1Name&lt;/param-name&gt;
 *                 &lt;param-value&gt;Node1Param1Value&lt;/param-value&gt;
 *             &lt;/param&gt;
 *             &lt;param id="n1-pid2"&gt;
 *                 &lt;param-name&gt;Param2Name&lt;/param-name&gt;
 *                 &lt;param-value&gt;Node1Param2Value&lt;/param-value&gt;
 *             &lt;/param&gt;
 *             &lt;!-- node 1.1 --&gt;
 *             &lt;menu-node id="id1_1"&gt;
 *                 &lt;display-info&gt;
 *                     &lt;locale&gt;en&lt;/locale&gt;
 *                     &lt;title&gt;Node 1.1&lt;/title&gt;
 *                     &lt;description&gt;Node 1.1 Description&lt;/description&gt;
 *                 &lt;/display-info&gt;
 *             &lt;/menu-node&gt; 
 *             &lt;!-- node 1.2 --&gt;
 *             &lt;menu-node id="id1_2"&gt;
 *                 &lt;display-info&gt;
 *                     &lt;locale&gt;en&lt;/locale&gt;
 *                     &lt;title&gt;Node 1.2&lt;/title&gt;
 *                     &lt;description&gt;Node 1.2 Description&lt;/description&gt;
 *                 &lt;/display-info&gt;
 *             &lt;/menu-node&gt;
 *         &lt;/menu-node&gt;
 *         &lt;!-- node 2 --&gt;
 *         &lt;menu-node id="id2"&gt;
 *             &lt;display-info&gt;
 *                 &lt;locale&gt;en&lt;/locale&gt;
 *                 &lt;title&gt;Node 2&lt;/title&gt;
 *                 &lt;description&gt;Node 2 Description&lt;/description&gt;
 *             &lt;/display-info&gt;
 *             &lt;display-info&gt;
 *                 &lt;locale&gt;fr&lt;/locale&gt;
 *                 &lt;title&gt;Noeud 2&lt;/title&gt;
 *                 &lt;description&gt;Noeud 2 Description&lt;/description&gt;
 *             &lt;/display-info&gt;
 *             &lt;action-name&gt;Node2Action&lt;/action-name&gt;
 *             &lt;param id="n2-pid1"&gt;
 *                 &lt;param-name&gt;Param1Name&lt;/param-name&gt;
 *                 &lt;param-value&gt;Node2Param1Value&lt;/param-value&gt;
 *             &lt;/param&gt;
 *             &lt;param id="n2-pid2"&gt;
 *                 &lt;param-name&gt;Param2Name&lt;/param-name&gt;
 *                 &lt;param-value&gt;Node2Param2Value&lt;/param-value&gt;
 *             &lt;/param&gt;
 *         &lt;/menu-node&gt; 
 *     &lt;/menu-node&gt;
 * &lt;/menu-tree&gt;
 * </PRE>
 * 
 * <P>Sample portlet source code:</P>
 * 
 * <PRE>
 * import java.io.*;
 * import org.apache.jetspeed.portlet.*;
 * import org.apache.jetspeed.portlet.event.*;
 * import com.ibm.wps.portlet.menu.*;
 * import com.ibm.wps.portlets.menu.*;
 * 
 * public class XMITestPortlet extends XMIMenuTreePortlet implements MenuProvider, ActionListener
 * {
 *     public void doView(PortletRequest request, PortletResponse response)
 *     throws PortletException, IOException
 *     {
 *         // Invoke the JSP to render
 *         getPortletConfig().getContext().include("/jsp/View.jsp", request, response);
 *     }
 * 
 *     public void actionPerformed(ActionEvent event) throws PortletException
 *     {
 *         if (event != null)
 *         {
 *             String actionString = event.getActionString();
 * 
 *             PortletRequest request = event.getRequest();
 * 
 *             if (actionString != null)
 *             {
 *                 request.setAttribute("ActionString", actionString);
 *             }
 *         }
 *     }
 * }
 * </PRE>
 * 
 * <P>with <CODE>/jsp/View.jsp</CODE>:</P>
 * 
 * <PRE>
 * &lt;%@ page contentType="text/html"%&gt;
 * &lt;H3&gt;Test of XMIMenuTree&lt;/H3&gt;
 * &lt;UL&gt;
 *  &lt;LI&gt;Expand root node and &lt;CITE&gt;Node 1&lt;/CITE&gt;.&lt;BR&gt;&lt;BR&gt;
 *  &lt;/LI&gt;
 *  &lt;LI&gt;Click &lt;CITE&gt;Node 1&lt;/CITE&gt; and &lt;CITE&gt;Node 2&lt;/CITE&gt;. Check if action string and request parameters are displayed.
 *      &lt;TABLE BORDER="0"&gt;
 *      &lt;TR&gt;
 *          &lt;TD&gt;Action:&lt;/TD&gt;
 *          &lt;TD&gt;&lt;%=request.getAttribute("ActionString")%&gt;&lt;/TD&gt;
 *      &lt;/TR&gt;&lt;TR&gt;
 *          &lt;TD&gt;Param1:&lt;/TD&gt;
 *          &lt;TD&gt;&lt;%=request.getParameter("Param1Name")%&gt;&lt;/TD&gt;
 *      &lt;/TR&gt;&lt;TR&gt;
 *          &lt;TD&gt;Param2:&lt;/TD&gt;
 *          &lt;TD&gt;&lt;%=request.getParameter("Param2Name")%&gt;&lt;/TD&gt;
 *      &lt;/TR&gt;
 *      &lt;/TABLE&gt;
 *      &lt;BR&gt;
 *  &lt;/LI&gt;
 * &lt;/UL&gt;
 * </PRE>
 * 
 * @ibm-spi
 * @since 5.0
 */
public class XMIMenuTreePortlet extends PortletAdapter
implements MenuProvider
{
    /** Copyright statement. */
    private static final String COPYRIGHT = com.ibm.wps.Copyright.SHORT;

    // This is a serializable class. All changes to this class need to be done 
    // on accordance to the Portal Policy for Serialization Compatability.
    // If you are not certain whether your changes to this class affect its 
    // serialization compatability, please consult any of the chief programmers.
    static final long serialVersionUID = 5100L;
    
    
    private static final String XMI_FILE_PATH = "XMIFilePath";

    // for global scope only: store tree
    private MenuTree menuTree = null;

    /**
     * Constructor.
     */
    protected XMIMenuTreePortlet()
    {
        super();
    }
    
    /**
     * Returns the menu tree containing the complete portlet menus configured in an XML file. 
     * 
     * @param menuContext
     *               contains the current request and response ID
     * 
     * @return the complete menu tree
     * 
     * @example This method is usually not extended.
     * Instead, the menu tree is configured in an XML file.
     * Its path is set in <CODE>portlet.xml</CODE>.
     * 
     * @exception PortletException
     *                   if XML file or service not found or unavailable or if XMIFilePath not specified
     * @see MenuProvider
     */
    public MenuTree getMenu(MenuContext menuContext) throws PortletException
    {
        // menu tree not yet created
        if ( menuTree == null )
        {
            PortletContext portletContext = getPortletConfig().getContext();

            // settings
            PortletSettings portletSettings = getPortletSettings();

            if ( portletSettings == null )
            {
                throw new PortletException ("XMIMenuTreePortlet: PortletSettings not found");
            }

            // file path
            String menuTreePath = (String) portletSettings.getAttribute(XMI_FILE_PATH);

            if ( menuTreePath == null )
            {
                throw new PortletException ("XMIMenuTreePortlet: Attribute XMIFilePath not found");
            }


            // open file
            InputStream inputStream = portletContext.getResourceAsStream(menuTreePath);

            if ( inputStream == null )
            {
                throw new PortletException ("XMIMenuTreePortlet: XML File '" + menuTreePath +
                                            "' not found");
            }

            InputStreamReader inputStreamReader = new InputStreamReader(inputStream);


            // services
            XMIMenuService service = null;

            try
            {
                service = (XMIMenuService) portletContext.getService(XMIMenuService.class);
            }
            catch ( org.apache.jetspeed.portlet.service.PortletServiceUnavailableException e )
            {
                throw new PortletException("XMIMenuTreePortlet: XMI Service is unavailable");
            }
            catch ( org.apache.jetspeed.portlet.service.PortletServiceNotFoundException e )
            {
                throw new PortletException("XMIMenuTreePortlet: XMI Service not found");
            }

            menuTree = service.getMenuTree(inputStreamReader, menuContext);

        }

        return(menuTree);
    }
}

